package kr.co.vividnext.sodalive.admin.audition.role

import com.fasterxml.jackson.databind.ObjectMapper
import kr.co.vividnext.sodalive.admin.audition.AdminAuditionRepository
import kr.co.vividnext.sodalive.audition.AuditionRole
import kr.co.vividnext.sodalive.aws.s3.S3Uploader
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.utils.generateFileName
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import org.springframework.web.multipart.MultipartFile

@Service
class AdminAuditionRoleService(
    private val s3Uploader: S3Uploader,
    private val objectMapper: ObjectMapper,
    private val repository: AdminAuditionRoleRepository,
    private val auditionRepository: AdminAuditionRepository,

    @Value("\${cloud.aws.s3.bucket}")
    private val bucket: String
) {
    @Transactional
    fun createAuditionRole(image: MultipartFile, requestString: String) {
        val request = objectMapper.readValue(requestString, CreateAuditionRoleRequest::class.java)
        val auditionRole = AuditionRole(
            name = request.name,
            information = request.information,
            auditionScriptUrl = request.auditionScriptUrl
        )
        val audition = auditionRepository.findByIdOrNull(id = request.auditionId)
            ?: throw SodaException("잘못된 요청입니다.\n다시 시도해 주세요.")
        auditionRole.audition = audition
        repository.save(auditionRole)

        val fileName = generateFileName("audition_role")
        val imagePath = s3Uploader.upload(
            inputStream = image.inputStream,
            bucket = bucket,
            filePath = "audition/role/${auditionRole.id}/$fileName"
        )
        auditionRole.imagePath = imagePath
    }

    @Transactional
    fun updateAuditionRole(image: MultipartFile?, requestString: String) {
        val request = objectMapper.readValue(requestString, UpdateAuditionRoleRequest::class.java)
        val auditionRole = repository.findByIdOrNull(id = request.id)
            ?: throw SodaException("잘못된 요청입니다.\n다시 시도해 주세요.")

        if (!request.name.isNullOrBlank()) {
            if (request.name.length < 2) throw SodaException("배역 이름은 최소 2글자 입니다")
            auditionRole.name = request.name
        }

        if (!request.information.isNullOrBlank()) {
            if (request.information.length < 10) throw SodaException("오디션 배역 정보는 최소 10글자 입니다")
            auditionRole.information = request.information
        }

        if (!request.auditionScriptUrl.isNullOrBlank()) {
            auditionRole.auditionScriptUrl = request.auditionScriptUrl
        }

        if (request.status != null) {
            auditionRole.status = request.status
        }

        if (request.isActive != null) {
            auditionRole.isActive = request.isActive
        }

        if (image != null) {
            val fileName = generateFileName("audition_role")
            val imagePath = s3Uploader.upload(
                inputStream = image.inputStream,
                bucket = bucket,
                filePath = "audition/role/${auditionRole.id}/$fileName"
            )
            auditionRole.imagePath = imagePath
        }
    }

    fun getAuditionRoleDetail(auditionRoleId: Long): GetAuditionRoleDetailResponse {
        return repository.getAuditionRoleDetail(auditionRoleId = auditionRoleId)
    }

    fun getAuditionApplicantList(auditionRoleId: Long, offset: Long, limit: Long): GetAuditionRoleApplicantResponse {
        val totalCount = repository.getAuditionApplicantTotalCount(auditionRoleId = auditionRoleId)
        val items = repository.getAuditionApplicantList(auditionRoleId = auditionRoleId, offset = offset, limit = limit)
        return GetAuditionRoleApplicantResponse(totalCount, items)
    }
}
