package kr.co.vividnext.sodalive.explorer.profile.creatorCommunity.comment

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.explorer.profile.creatorCommunity.comment.QCreatorCommunityComment.creatorCommunityComment
import org.springframework.data.jpa.repository.JpaRepository
import java.time.ZoneId
import java.time.format.DateTimeFormatter

interface CreatorCommunityCommentRepository :
    JpaRepository<CreatorCommunityComment, Long>, CreatorCommunityCommentQueryRepository

interface CreatorCommunityCommentQueryRepository {
    fun findByPostId(
        cloudFrontHost: String,
        timezone: String,
        id: Long,
        offset: Long,
        limit: Long
    ): List<GetCommunityPostCommentListItem>

    fun commentReplyCountByCommentId(commentId: Long): Int

    fun totalCountCommentByPostId(postId: Long): Int

    fun getCommunityCommentReplyList(
        cloudFrontHost: String,
        commentId: Long,
        timezone: String,
        offset: Long,
        limit: Long
    ): List<GetCommunityPostCommentListItem>
}

class CreatorCommunityCommentQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : CreatorCommunityCommentQueryRepository {
    override fun findByPostId(
        cloudFrontHost: String,
        timezone: String,
        id: Long,
        offset: Long,
        limit: Long
    ): List<GetCommunityPostCommentListItem> {
        return queryFactory
            .selectFrom(creatorCommunityComment)
            .where(
                creatorCommunityComment.isActive.isTrue
                    .and(creatorCommunityComment.creatorCommunity.id.eq(id))
                    .and(creatorCommunityComment.parent.isNull)
            )
            .offset(offset)
            .limit(limit)
            .orderBy(creatorCommunityComment.createdAt.desc())
            .fetch()
            .asSequence()
            .map {
                val date = it.createdAt!!
                    .atZone(ZoneId.of("UTC"))
                    .withZoneSameInstant(ZoneId.of(timezone))

                GetCommunityPostCommentListItem(
                    id = it.id!!,
                    writerId = it.member!!.id!!,
                    nickname = it.member!!.nickname,
                    profileUrl = if (it.member!!.profileImage != null) {
                        "$cloudFrontHost/${it.member!!.profileImage}"
                    } else {
                        "$cloudFrontHost/profile/default-profile.png"
                    },
                    comment = it.comment,
                    date = date.format(DateTimeFormatter.ofPattern("yyyy.MM.dd E hh:mm a")),
                    replyCount = commentReplyCountByCommentId(it.id!!)
                )
            }
            .toList()
    }

    override fun commentReplyCountByCommentId(commentId: Long): Int {
        return queryFactory.select(creatorCommunityComment.id)
            .from(creatorCommunityComment)
            .where(
                creatorCommunityComment.isActive.isTrue
                    .and(creatorCommunityComment.parent.isNotNull)
                    .and(creatorCommunityComment.parent.id.eq(commentId))
            )
            .fetch()
            .size
    }

    override fun totalCountCommentByPostId(postId: Long): Int {
        return queryFactory.select(creatorCommunityComment.id)
            .from(creatorCommunityComment)
            .where(
                creatorCommunityComment.creatorCommunity.id.eq(postId)
                    .and(creatorCommunityComment.isActive.isTrue)
                    .and(creatorCommunityComment.parent.isNull)
            )
            .fetch()
            .size
    }

    override fun getCommunityCommentReplyList(
        cloudFrontHost: String,
        commentId: Long,
        timezone: String,
        offset: Long,
        limit: Long
    ): List<GetCommunityPostCommentListItem> {
        return queryFactory
            .selectFrom(creatorCommunityComment)
            .where(
                creatorCommunityComment.isActive.isTrue
                    .and(creatorCommunityComment.parent.isNotNull)
                    .and(creatorCommunityComment.parent.id.eq(commentId))
            )
            .offset(offset)
            .limit(limit)
            .orderBy(creatorCommunityComment.createdAt.desc())
            .fetch()
            .asSequence()
            .map {
                val date = it.createdAt!!
                    .atZone(ZoneId.of("UTC"))
                    .withZoneSameInstant(ZoneId.of(timezone))

                GetCommunityPostCommentListItem(
                    id = it.id!!,
                    writerId = it.member!!.id!!,
                    nickname = it.member!!.nickname,
                    profileUrl = if (it.member!!.profileImage != null) {
                        "$cloudFrontHost/${it.member!!.profileImage}"
                    } else {
                        "$cloudFrontHost/profile/default-profile.png"
                    },
                    comment = it.comment,
                    date = date.format(DateTimeFormatter.ofPattern("yyyy.MM.dd E hh:mm a")),
                    replyCount = commentReplyCountByCommentId(it.id!!)
                )
            }
            .toList()
    }
}
